#ifndef _IPQEXT_H
#define _IPQEXT_H

#define IPQEXT_NAME_LEN     255        // maximum length of extension name
#define IPQEXT_PARAM_LEN    1023       // maximum length of parameter buffer

/* ipqext structure */
typedef struct ipqext ipqext_t;
struct ipqext{
  struct ipqext *prev, *next;          // ipqext chain links
  void* handle;                        // dynamic module handle
  int active;                          // state: started / stopped

  unsigned long groupmask;             // group mask
  const char *name;                    // extension name
  const char *version;                 // extension version
  int (*init)();                       // extension initialization function
  int (*fini)();                       // extension finalization function
  int (*proc)(ipq_packet_msg_t *m);    // packet processing function
  int (*start)();                      // extension start function
  int (*stop)();                       // extension stop function
  int (*reset)();                      // extension reset function
  const char* (*info)();               // extension information function
  const char* (*help)();               // extension helpfunction
};

/* packet layout */
struct pkt_layout {
  struct iphdr *iph;                   // IP header pointer
  int iph_len;                         // IP header length
  void *upph;                          // upper layer header pointer
  int upph_len;                        // upper layer header length
  unsigned char *data;                 // data payload pointer
  int data_len;                        // data payload length
};

#ifdef _IPQEXT_EXTENSION_
int init();
int fini();
int proc(ipq_packet_msg_t *m);
int start();
int stop();
int reset();
const char *info();
const char *help();
int init_pass(void *handle, char *param, int paramlen);

extern void ipqext_register(ipqext_t *ipqext);
extern void ipqext_unregister(ipqext_t *ipqext);
#endif

#endif /* _IPQEXT_H */
